@@ -25,6 +25,7 @@ static char rcsid[] = "@(#$Id: rclconfig.cpp,v 1.62 2008-10-08 16:15:22 dockes E
 #include <langinfo.h>
 
 #include <set>
+#include <algorithm>
 
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -203,6 +204,12 @@ bool RclConfig::updateMainConfig()
 	    TextSplit::cjkProcessing(true);
 	}
     }
+
+    bool nonum = false;
+    if (getConfParam("nonumbers", &nonum) && nonum == true) {
+	TextSplit::noNumbers();
+    }
+
     m_skpnstate.init(this, m_conf, "skippedNames");
     m_rmtstate.init(this, m_conf, "indexedmimetypes");
     return true;
@@ -263,19 +270,35 @@ bool RclConfig::getConfParam(const std::string &name, bool *bvp)
     return true;
 }
 
+bool RclConfig::getConfParam(const std::string &name, vector<string> *svvp)
+{
+    if (!svvp) 
+	return false;
+    svvp->clear();
+    string s;
+    if (!getConfParam(name, s))
+	return false;
+    return stringToStrings(s, *svvp);
+}
+bool RclConfig::getConfParam(const std::string &name, list<string> *svvp)
+{
+    if (!svvp) 
+	return false;
+    svvp->clear();
+    string s;
+    if (!getConfParam(name, s))
+	return false;
+    return stringToStrings(s, *svvp);
+}
+
 list<string> RclConfig::getTopdirs()
 {
     list<string> tdl;
-    // Retrieve the list of directories to be indexed.
-    string topdirs;
-    if (!getConfParam("topdirs", topdirs)) {
-	LOGERR(("RclConfig::getTopdirs: no top directories in config\n"));
-	return tdl;
-    }
-    if (!stringToStrings(topdirs, tdl)) {
-	LOGERR(("RclConfig::getTopdirs: parse error for directory list\n"));
+    if (!getConfParam("topdirs", &tdl)) {
+	LOGERR(("RclConfig::getTopdirs: no top directories in config or bad list format\n"));
 	return tdl;
     }
+
     for (list<string>::iterator it = tdl.begin(); it != tdl.end(); it++) {
 	*it = path_tildexpand(*it);
 	*it = path_canon(*it);
@@ -532,7 +555,9 @@ void RclConfig::storeMissingHelperDesc(const string &s)
     string fmiss = path_cat(getConfDir(), "missing");
     FILE *fp = fopen(fmiss.c_str(), "w");
     if (fp) {
-	fwrite(s.c_str(), s.size(), 1, fp);
+	if (fwrite(s.c_str(), s.size(), 1, fp) != 1) {
+            LOGERR(("storeMissingHelperDesc: fwrite failed\n"));
+        }
 	fclose(fp);
     }
 }
@@ -584,10 +609,8 @@ bool RclConfig::readFieldsConfig(const string& cnferrloc)
     }
 #endif
 
-    string ss;
-    if (m_fields->get("stored", ss, "stored")) {
-	list<string> sl;
-	stringToStrings(ss, sl);
+    list<string> sl = m_fields->getNames("stored");
+    if (!sl.empty()) {
 	for (list<string>::const_iterator it = sl.begin(); 
 	     it != sl.end(); it++) {
 	    string fld = fieldCanon(stringtolower(*it));
@@ -629,6 +652,8 @@ bool RclConfig::getFieldPrefix(const string& _fld, string &pfx)
 bool RclConfig::getFieldSpecialisations(const string& fld, 
 					list<string>& children, bool top)
 {
+    if (m_fields == 0)
+        return false;
     string sclds;
     children.push_back(fld);
     if (m_fields->get(fld, sclds, "specialisations")) {
@@ -676,11 +701,29 @@ string RclConfig::fieldCanon(const string& f)
     return fld;
 }
 
+list<string> RclConfig::getFieldSectNames(const string &sk, const char* patrn)
+{
+    if (m_fields == 0)
+        return list<string>();
+    return m_fields->getNames(sk, patrn);
+}
+
+bool RclConfig::getFieldConfParam(const string &name, const string &sk, 
+                                  string &value)
+{
+    if (m_fields == 0)
+        return false;
+    return m_fields->get(name, value, sk);
+}
+
+
 string RclConfig::getMimeViewerDef(const string &mtype, const string& apptag)
 {
     LOGDEB(("RclConfig::getMimeViewerDef: mtype %s apptag %s\n",
             mtype.c_str(), apptag.c_str()));
     string hs;
+    if (mimeview == 0)
+	return hs;
     if (apptag.empty() || !mimeview->get(mtype + string("|") + apptag,
                                          hs, "view"))
         mimeview->get(mtype, hs, "view");
@@ -700,6 +743,8 @@ bool RclConfig::getMimeViewerDefs(vector<pair<string, string> >& defs)
 
 bool RclConfig::setMimeViewerDef(const string& mt, const string& def)
 {
+    if (mimeview == 0)
+        return false;
     string pconfname = path_cat(m_confdir, "mimeview");
     // Make sure this exists 
     close(open(pconfname.c_str(), O_CREAT|O_WRONLY, 0600));
@@ -723,6 +768,17 @@ bool RclConfig::setMimeViewerDef(const string& mt, const string& def)
     return true;
 }
 
+bool RclConfig::mimeViewerNeedsUncomp(const string &mimetype)
+{
+    string s;
+    vector<string> v;
+    if (mimeview != 0 && mimeview->get("nouncompforviewmts", s, "") && 
+        stringToStrings(s, v) && 
+        find_if(v.begin(), v.end(), StringIcmpPred(mimetype)) != v.end()) 
+        return false;
+    return true;
+}
+
 /**
  * Return icon name and path
  */
@@ -785,10 +841,8 @@ list<string>& RclConfig::getSkippedNames()
 list<string> RclConfig::getSkippedPaths()
 {
     list<string> skpl;
-    string skipped;
-    if (getConfParam("skippedPaths", skipped)) {
-	stringToStrings(skipped, skpl);
-    }
+    getConfParam("skippedPaths", &skpl);
+
     // Always add the dbdir and confdir to the skipped paths. This is 
     // especially important for the rt monitor which will go into a loop if we
     // don't do this.
@@ -808,10 +862,7 @@ list<string> RclConfig::getDaemSkippedPaths()
     list<string> skpl = getSkippedPaths();
 
     list<string> dskpl;
-    string skipped;
-    if (getConfParam("daemSkippedPaths", skipped)) {
-	stringToStrings(skipped, dskpl);
-    }
+    getConfParam("daemSkippedPaths", &dskpl);
 
     for (list<string>::iterator it = dskpl.begin(); it != dskpl.end(); it++) {
 	*it = path_tildexpand(*it);
@@ -991,7 +1042,6 @@ void RclConfig::initFrom(const RclConfig& r)
 
 #include <stdio.h>
 #include <signal.h>
-#include <sys/stat.h>
 
 #include <iostream>
 #include <list>
