@@ -97,9 +97,13 @@ class RclConfig {
 	return m_conf->get(name, value, m_keydir);
     }
     /** Variant with autoconversion to int */
-    bool getConfParam(const std::string &name, int *value);
+    bool getConfParam(const string &name, int *value);
     /** Variant with autoconversion to bool */
-    bool getConfParam(const std::string &name, bool *value);
+    bool getConfParam(const string &name, bool *value);
+    /** Variant with conversion to string list/vector
+     *  (stringToStrings). Can fail if the string is malformed. */
+    bool getConfParam(const string &name, vector<string> *value);
+    bool getConfParam(const string &name, list<string> *value);
 
     /** 
      * Get list of config names under current sk, with possible 
@@ -195,11 +199,20 @@ class RclConfig {
     string fieldCanon(const string& fld);
     /** Get xattr name to field names translations */
     const map<string, string>& getXattrToField() {return m_xattrtofld;}
-    
+    /** Get value of a parameter inside the "fields" file. Only some filters 
+        use this (ie: mh_mail). The information specific to a given filter
+        is typically stored in a separate section(ie: [mail]) */
+    list<string> getFieldSectNames(const string &sk, const char* = 0);
+    bool getFieldConfParam(const string &name, const string &sk, string &value);
+
     /** mimeview: get/set external viewer exec string(s) for mimetype(s) */
     string getMimeViewerDef(const string &mimetype, const string& apptag);
     bool getMimeViewerDefs(vector<pair<string, string> >&);
     bool setMimeViewerDef(const string& mimetype, const string& cmd);
+    /** Check if mime type is designated as needing no uncompress before view
+     * (if a file of this type is found compressed). Default is true,
+     *  exceptions are found in the nouncompforviewmts mimeview list */
+    bool mimeViewerNeedsUncomp(const string &mimetype);
 
     /** Store/retrieve missing helpers description string */
     string getMissingHelperDesc();
