@@ -67,26 +67,10 @@ using namespace std;
 #endif
 
 FsIndexer::~FsIndexer() {
-    // Maybe clean up temporary directory
-    if (!m_tmpdir.empty()) {
-	wipedir(m_tmpdir);
-	if (rmdir(m_tmpdir.c_str()) < 0) {
-	    LOGERR(("FsIndexer::~FsIndexer: cannot clear temp dir %s\n",
-		    m_tmpdir.c_str()));
-	}
-    }
 }
 
 bool FsIndexer::init()
 {
-    if (m_tmpdir.empty() || access(m_tmpdir.c_str(), 0) < 0) {
-	if (!maketmpdir(m_tmpdir, m_reason)) {
-	    LOGERR(("FsIndexer: cannot create temporary directory: %s\n",
-		    m_reason.c_str()));
-            m_tmpdir.erase();
-	    return false;
-	}
-    }
     if (m_tdl.empty()) {
         m_tdl = m_config->getTopdirs();
         if (m_tdl.empty()) {
@@ -388,8 +372,13 @@ FsIndexer::processone(const std::string &fn, const struct stat *stp,
              displayableBytes(stp->st_size).c_str(), fn.c_str()));
 
     FileInterner interner(fn, stp, m_config, m_tmpdir, FileInterner::FIF_none);
+    if (!interner.ok()) {
+        // no indexing whatsoever in this case. This typically means that
+        // indexallfilenames is not set
+        return FsTreeWalker::FtwOk;
+    }
 
-    // File name transcoded to utf8 for indexation. 
+    // File name transcoded to utf8 for indexing. 
     string charset = m_config->getDefCharset(true);
     // If this fails, the file name won't be indexed, no big deal
     // Note that we used to do the full path here, but I ended up believing
@@ -474,6 +463,8 @@ FsIndexer::processone(const std::string &fn, const struct stat *stp,
 	// Tell what we are doing and check for interrupt request
 	if (m_updater) {
 	    ++(m_updater->status.docsdone);
+            if (m_updater->status.dbtotdocs < m_updater->status.docsdone)
+                m_updater->status.dbtotdocs = m_updater->status.docsdone;
             m_updater->status.fn = fn;
             if (!ipath.empty())
                 m_updater->status.fn += "|" + ipath;
