@@ -21,6 +21,7 @@ static char rcsid[] = "@(#$Id: internfile.cpp,v 1.46 2008-10-10 08:04:54 dockes
 #ifndef TEST_INTERNFILE
 #include "autoconfig.h"
 
+#include <stdio.h>
 #include <unistd.h>
 #include <fcntl.h>
 #include <sys/types.h>
@@ -41,12 +42,12 @@ using namespace std;
 #include "mimehandler.h"
 #include "execmd.h"
 #include "pathut.h"
-#include "wipedir.h"
 #include "rclconfig.h"
 #include "mh_html.h"
 #include "fileudi.h"
-#include "beaglequeue.h"
+#include "beaglequeuecache.h"
 #include "cancelcheck.h"
+#include "copyfile.h"
 
 #ifdef RCL_USE_XATTR
 #include "pxattr.h"
@@ -111,12 +112,12 @@ bool FileInterner::getEnclosing(const string &url, const string &ipath,
 // Uncompress input file into a temporary one, by executing the appropriate
 // script.
 static bool uncompressfile(RclConfig *conf, const string& ifn, 
-			   const list<string>& cmdv, const string& tdir, 
+			   const list<string>& cmdv, TempDir& tdir, 
 			   string& tfile)
 {
     // Make sure tmp dir is empty. we guarantee this to filters
-    if (wipedir(tdir) != 0) {
-	LOGERR(("uncompressfile: can't clear temp dir %s\n", tdir.c_str()));
+    if (!tdir.ok() || !tdir.wipe()) {
+	LOGERR(("uncompressfile: can't clear temp dir %s\n", tdir.dirname()));
 	return false;
     }
     string cmd = cmdv.front();
@@ -127,7 +128,7 @@ static bool uncompressfile(RclConfig *conf, const string& ifn,
     list<string> args;
     map<char, string> subs;
     subs['f'] = ifn;
-    subs['t'] = tdir;
+    subs['t'] = tdir.dirname();
     for (; it != cmdv.end(); it++) {
 	string ns;
 	pcSubst(*it, ns, subs);
@@ -140,7 +141,7 @@ static bool uncompressfile(RclConfig *conf, const string& ifn,
     if (status || tfile.empty()) {
 	LOGERR(("uncompressfile: doexec: failed for [%s] status 0x%x\n", 
 		ifn.c_str(), status));
-	if (wipedir(tdir.c_str())) {
+	if (!tdir.wipe()) {
 	    LOGERR(("uncompressfile: wipedir failed\n"));
 	}
 	return false;
@@ -153,7 +154,7 @@ static bool uncompressfile(RclConfig *conf, const string& ifn,
 // Delete temporary uncompressed file
 void FileInterner::tmpcleanup()
 {
-    if (m_tdir.empty() || m_tfile.empty())
+    if (m_tfile.empty())
 	return;
     if (unlink(m_tfile.c_str()) < 0) {
 	LOGERR(("FileInterner::tmpcleanup: unlink(%s) errno %d\n", 
@@ -171,15 +172,15 @@ void FileInterner::tmpcleanup()
 // Split into "constructor calls init()" to allow use from other constructor
 FileInterner::FileInterner(const string &f, const struct stat *stp,
 			   RclConfig *cnf, 
-			   const string& td, int flags, const string *imime)
-    : m_tdir(td)
+			   TempDir& td, int flags, const string *imime)
+    : m_tdir(td), m_ok(false)
 {
     initcommon(cnf, flags);
-    init(f, stp, cnf, td, flags, imime);
+    init(f, stp, cnf, flags, imime);
 }
 
 void FileInterner::init(const string &f, const struct stat *stp, RclConfig *cnf,
-                        const string& td, int flags, const string *imime)
+                        int flags, const string *imime)
 {
     m_fn = f;
 
@@ -197,12 +198,12 @@ void FileInterner::init(const string &f, const struct stat *stp, RclConfig *cnf,
 
     if (flags & FIF_doUseInputMimetype) {
         if (!imime) {
-            LOGERR(("FileInterner::FileInterner: told to use null imime\n"));
+            LOGERR(("FileInterner:: told to use null imime\n"));
             return;
         }
         l_mime = *imime;
     } else {
-        LOGDEB(("FileInterner::FileInterner: [%s] mime [%s] preview %d\n", 
+        LOGDEB(("FileInterner:: [%s] mime [%s] preview %d\n", 
                 f.c_str(), imime?imime->c_str() : "(null)", m_forPreview));
 
         // We need to run mime type identification in any case to check
@@ -229,15 +230,15 @@ void FileInterner::init(const string &f, const struct stat *stp, RclConfig *cnf,
 		if (!uncompressfile(m_cfg, m_fn, ucmd, m_tdir, m_tfile)) {
 		    return;
 		}
-		LOGDEB1(("internfile: after ucomp: m_tdir %s, tfile %s\n", 
-			 m_tdir.c_str(), m_tfile.c_str()));
+		LOGDEB1(("FileInterner:: after ucomp: m_tdir %s, tfile %s\n", 
+			 m_tdir.dirname(), m_tfile.c_str()));
 		m_fn = m_tfile;
 		// Note: still using the original file's stat. right ?
 		l_mime = mimetype(m_fn, stp, m_cfg, usfci);
 		if (l_mime.empty() && imime)
 		    l_mime = *imime;
 	    } else {
-		LOGINFO(("internfile: %s over size limit %d kbs\n",
+		LOGINFO(("FileInterner:: %s over size limit %d kbs\n",
 			 m_fn.c_str(), maxkbs));
 	    }
 	}
@@ -246,7 +247,7 @@ void FileInterner::init(const string &f, const struct stat *stp, RclConfig *cnf,
     if (l_mime.empty()) {
 	// No mime type. We let it through as config may warrant that
 	// we index all file names
-	LOGDEB(("internfile: (no mime) [%s]\n", m_fn.c_str()));
+	LOGDEB0(("FileInterner:: no mime: [%s]\n", m_fn.c_str()));
     }
 
     // Look for appropriate handler (might still return empty)
@@ -257,7 +258,7 @@ void FileInterner::init(const string &f, const struct stat *stp, RclConfig *cnf,
 	// No handler for this type, for now :( if indexallfilenames
 	// is set in the config, this normally wont happen (we get mh_unknown)
 	LOGINFO(("FileInterner:: ignored: [%s] mime [%s]\n", 
-                 f.c_str(), l_mime.c_str()));
+                f.c_str(), l_mime.c_str()));
 	return;
     }
     df->set_property(Dijon::Filter::OPERATING_MODE, 
@@ -275,26 +276,26 @@ void FileInterner::init(const string &f, const struct stat *stp, RclConfig *cnf,
 #endif //RCL_USE_XATTR
 
     if (!df->set_document_file(m_fn)) {
-	LOGERR(("FileInterner:: error parsing %s\n", m_fn.c_str()));
+	LOGERR(("FileInterner:: error converting %s\n", m_fn.c_str()));
 	return;
     }
 
     m_handlers.push_back(df);
-    LOGDEB(("FileInterner::FileInterner: %s [%s]\n", l_mime.c_str(), 
-	     m_fn.c_str()));
+    LOGDEB(("FileInterner:: init ok %s [%s]\n", l_mime.c_str(), m_fn.c_str()));
+    m_ok = true;
 }
 
 // Setup from memory data (ie: out of the web cache). imime needs to be set.
 FileInterner::FileInterner(const string &data, RclConfig *cnf, 
-                           const string& td, int flags, const string& imime)
-    : m_tdir(td)
+                           TempDir& td, int flags, const string& imime)
+    : m_tdir(td), m_ok(false)
 {
     initcommon(cnf, flags);
-    init(data, cnf, td, flags, imime);
+    init(data, cnf, flags, imime);
 }
 
 void FileInterner::init(const string &data, RclConfig *cnf, 
-                        const string& td, int flags, const string& imime)
+                        int flags, const string& imime)
 {
     if (imime.empty()) {
 	LOGERR(("FileInterner: inmemory constructor needs input mime type\n"));
@@ -338,6 +339,7 @@ void FileInterner::init(const string &data, RclConfig *cnf,
 	return;
     }
     m_handlers.push_back(df);
+    m_ok = true;
 }
 
 void FileInterner::initcommon(RclConfig *cnf, int flags)
@@ -352,8 +354,8 @@ void FileInterner::initcommon(RclConfig *cnf, int flags)
 }
 
 FileInterner::FileInterner(const Rcl::Doc& idoc, RclConfig *cnf, 
-                           const string& td, int flags)
-    : m_tdir(td)
+                           TempDir& td, int flags)
+    : m_tdir(td), m_ok(false)
 {
     LOGDEB(("FileInterner::FileInterner(idoc)\n"));
     initcommon(cnf, flags);
@@ -388,16 +390,12 @@ FileInterner::FileInterner(const Rcl::Doc& idoc, RclConfig *cnf,
                     fn.c_str()));
             return;
         }
-        init(fn, &st, cnf, td, flags, &idoc.mimetype);
+        init(fn, &st, cnf, flags, &idoc.mimetype);
     } else if (!backend.compare("BGL")) {
-        // Retrieve from our webcache (beagle data). There should
-        // probably be a separate object type for readonly cache
-        // access (distinct from the one used for indexing). 
-        // Anyway, we're not called in the same thread as indexing ops, and
-        // even, at worse, this would duplicate the memory used. The beagler
-        // object is created at the first call of this routine and deleted 
-        // when the program exits.
-        static BeagleQueueIndexer beagler(cnf);
+        // Retrieve from our webcache (beagle data). The beagler
+        // object is created at the first call of this routine and
+        // deleted when the program exits.
+        static BeagleQueueCache beagler(cnf);
         string data;
         Rcl::Doc dotdoc;
         map<string,string>::const_iterator it = 
@@ -416,7 +414,7 @@ FileInterner::FileInterner(const Rcl::Doc& idoc, RclConfig *cnf,
             LOGINFO(("FileInterner:: udi [%s], mimetp mismatch: in: [%s], bgl "
                      "[%s]\n", idoc.mimetype.c_str(), dotdoc.mimetype.c_str()));
         }
-        init(data, cnf, td, flags, dotdoc.mimetype);
+        init(data, cnf, flags, dotdoc.mimetype);
     } else {
         LOGERR(("FileInterner:: unknown backend: [%s]\n", backend.c_str()));
         return;
@@ -756,7 +754,7 @@ FileInterner::Status FileInterner::internfile(Rcl::Doc& doc, string& ipath)
     vector<string> vipath(MAXHANDLERS);
     int vipathidx = 0;
     if (!ipath.empty()) {
-	list<string> lipath;
+	vector<string> lipath;
 	stringToTokens(ipath, lipath, isep, true);
 	vipath.insert(vipath.begin(), lipath.begin(), lipath.end());
 	if (!m_handlers.back()->skip_to_document(vipath[m_handlers.size()-1])){
@@ -870,20 +868,6 @@ FileInterner::Status FileInterner::internfile(Rcl::Doc& doc, string& ipath)
 	return FIAgain;
 }
 
-// Automatic cleanup of iDocToFile's temp dir
-class DirWiper {
- public:
-    string dir;
-    bool do_it;
-    DirWiper(string d) : dir(d), do_it(true) {}
-    ~DirWiper() {
-	if (do_it) {
-	    wipedir(dir);
-	    rmdir(dir.c_str());
-	}
-    }
-};
-
 // Temporary while we fix backend things
 static string urltolocalpath(string url)
 {
@@ -896,8 +880,7 @@ static string urltolocalpath(string url)
 // the input mtype, so that no data conversion is performed.
 // We then write the data out of the resulting document into the output file.
 // There are two temporary objects:
-// - The internfile temporary directory gets destroyed before we
-//   return by the DirWiper object
+// - The internfile temporary directory gets destroyed by its destructor
 // - The output temporary file which is held in a reference-counted
 //   object and will be deleted when done with.
 bool FileInterner::idocToFile(TempFile& otemp, const string& tofile,
@@ -905,37 +888,38 @@ bool FileInterner::idocToFile(TempFile& otemp, const string& tofile,
 {
     LOGDEB(("FileInterner::idocToFile\n"));
     idoc.dump();
-    string fn = urltolocalpath(idoc.url);
-    string ipath = idoc.ipath;
-    string mtype = idoc.mimetype;
-    struct stat st;
-    if (stat(fn.c_str(), &st) < 0) {
-	LOGERR(("FileInterner::idocToFile: can't stat [%s]\n", fn.c_str()));
-	return false;
-    }
 
-    string tmpdir, reason;
-    if (!maketmpdir(tmpdir, reason))
-	return false;
-    DirWiper wiper(tmpdir);
+    TempDir tmpdir;
 
     // We set FIF_forPreview for consistency with the previous version
     // which determined this by looking at mtype!=null. Probably
     // doesn't change anything in this case.
-    FileInterner interner(fn, &st, cnf, tmpdir, FIF_forPreview, &mtype);
-    interner.setTargetMType(mtype);
+    FileInterner interner(idoc, cnf, tmpdir, FIF_forPreview);
+    interner.setTargetMType(idoc.mimetype);
     Rcl::Doc doc;
-    string mipath = ipath;
+    string mipath = idoc.ipath;
     Status ret = interner.internfile(doc, mipath);
     if (ret == FileInterner::FIError) {
 	LOGERR(("FileInterner::idocToFile: internfile() failed\n"));
 	return false;
     }
 
+    // Specialcase text/html. This is to work around a bug that will
+    // get fixed some day: internfile initialisation does not check
+    // targetmtype, so that at least one conversion is always
+    // performed. A common case would be an "Open" on an html file
+    // (we'd end up with text/plain content). As the html version is
+    // saved in this case, use it.  
+    if (!stringlowercmp("text/html", idoc.mimetype) && 
+        !interner.get_html().empty()) {
+        doc.text = interner.get_html();
+        doc.mimetype = "text/html";
+    }
+
     string filename;
     TempFile temp;
     if (tofile.empty()) {
-	TempFile temp1(new TempFileInternal(cnf->getSuffixFromMimeType(mtype)));
+	TempFile temp1(new TempFileInternal(cnf->getSuffixFromMimeType(idoc.mimetype)));
 	temp = temp1;
 	if (!temp->ok()) {
 	    LOGERR(("FileInterner::idocToFile: cant create temporary file"));
@@ -964,6 +948,110 @@ bool FileInterner::idocToFile(TempFile& otemp, const string& tofile,
     return true;
 }
 
+bool FileInterner::isCompressed(const string& fn, RclConfig *cnf)
+{
+    LOGDEB(("FileInterner::isCompressed: [%s]\n", fn.c_str()));
+    struct stat st;
+    if (stat(fn.c_str(), &st) < 0) {
+        LOGERR(("FileInterner::isCompressed: can't stat [%s]\n", fn.c_str()));
+        return false;
+    }
+    string l_mime = mimetype(fn, &st, cnf, true);
+    if (l_mime.empty()) {
+        LOGERR(("FileInterner::isUncompressed: can't get mime for [%s]\n", 
+                fn.c_str()));
+        return false;
+    }
+
+    list<string>ucmd;
+    if (cnf->getUncompressor(l_mime, ucmd)) {
+        return true;
+    }
+    return false;
+}
+
+bool FileInterner::maybeUncompressToTemp(TempFile& temp, const string& fn, 
+                                         RclConfig *cnf, const Rcl::Doc& doc)
+{
+    LOGDEB(("FileInterner::maybeUncompressToTemp: [%s]\n", fn.c_str()));
+    struct stat st;
+    if (stat(fn.c_str(), &st) < 0) {
+        LOGERR(("FileInterner::maybeUncompressToTemp: can't stat [%s]\n", 
+                fn.c_str()));
+        return false;
+    }
+    string l_mime = mimetype(fn, &st, cnf, true);
+    if (l_mime.empty()) {
+        LOGERR(("FileInterner::maybeUncompress.: can't id. mime for [%s]\n", 
+                fn.c_str()));
+        return false;
+    }
+
+    list<string>ucmd;
+    if (!cnf->getUncompressor(l_mime, ucmd)) {
+        return true;
+    }
+    // Check for compressed size limit
+    int maxkbs = -1;
+    if (cnf->getConfParam("compressedfilemaxkbs", &maxkbs) &&
+        maxkbs >= 0 && int(st.st_size / 1024) > maxkbs) {
+        LOGINFO(("FileInterner:: %s over size limit %d kbs\n",
+                 fn.c_str(), maxkbs));
+        return false;
+    }
+    TempDir tmpdir;
+    temp = 
+      TempFile(new TempFileInternal(cnf->getSuffixFromMimeType(doc.mimetype)));
+    if (!tmpdir.ok() || !temp->ok()) {
+        LOGERR(("FileInterner: cant create temporary file/dir"));
+        return false;
+    }
+
+    // uncompressfile choses the output file name, there is good
+    // reason for this, but it's not nice here. Have to copy or rename
+    // the uncompressed file
+    string uncomped;
+    if (!uncompressfile(cnf, fn, ucmd, tmpdir, uncomped)) {
+        return false;
+    }
+
+    // uncompressfile choses the output file name, there is good
+    // reason for this, but it's not nice here. Have to copy or rename
+    // the uncompressed file. 
+    // Hopefully the cross-dev case won't happen as we're 
+    // probably choosing the temp names in the same dir. However...
+    // Unix really should have a rename-else-copy call... 
+    if (stat(temp->filename(), &st) < 0) {
+        LOGERR(("FileInterner::maybeUncompressToTemp: can't stat [%s]\n", 
+                temp->filename()));
+        return false;
+    }
+    struct stat st1;
+    if (stat(uncomped.c_str(), &st1) < 0) {
+        LOGERR(("FileInterner::maybeUncompressToTemp: can't stat [%s]\n", 
+                uncomped.c_str()));
+        return false;
+    }
+    if (st.st_dev == st1.st_dev) {
+        if (rename(uncomped.c_str(), temp->filename()) < 0) {
+            LOGERR(("FileInterner::maybeUncompress: rename [%s] -> [%s]"
+                    "failed, errno %d\n", 
+                    uncomped.c_str(), temp->filename(), errno));
+            return false;
+        }
+    } else {
+        string reason;
+        bool ret = copyfile(uncomped.c_str(), temp->filename(), reason);
+        if (ret == false) {
+            LOGERR(("FileInterner::maybeUncompress: copy [%s] -> [%s]"
+                    "failed: %s\n", 
+                    uncomped.c_str(), temp->filename(), reason.c_str()));
+            return false;
+        }
+        // We let the tempdir cleanup get rid of uncomped
+    }
+    return true;
+}
 
 #else
 
