@@ -102,14 +102,14 @@ bool MimeHandlerExecMultiple::readDataElement(string& name, string &data)
     }
 
     // We're expecting something like Name: len\n
-    list<string> tokens;
+    vector<string> tokens;
     stringToTokens(ibuf, tokens);
     if (tokens.size() != 2) {
         LOGERR(("MHExecMultiple: bad line in filter output: [%s]\n",
                 ibuf.c_str()));
         return false;
     }
-    list<string>::iterator it = tokens.begin();
+    vector<string>::iterator it = tokens.begin();
     name = *it++;
     string& slen = *it;
     int len;
@@ -169,8 +169,10 @@ bool MimeHandlerExecMultiple::next_document()
     if (m_ipath.length()) {
         obuf << "Ipath: " << m_ipath.length() << "\n" << m_ipath;
     }
+    obuf << "Mimetype: " << m_mimeType.length() << "\n" << m_mimeType;
     obuf << "\n";
     if (m_cmd.send(obuf.str()) < 0) {
+        m_cmd.zapChild();
         LOGERR(("MHExecMultiple: send error\n"));
         return false;
     }
@@ -184,6 +186,7 @@ bool MimeHandlerExecMultiple::next_document()
     for (int loop=0;;loop++) {
         string name, data;
         if (!readDataElement(name, data)) {
+            m_cmd.zapChild();
             return false;
         }
         if (name.empty())
