@@ -18,6 +18,8 @@ static char rcsid[] = "@(#$Id: mh_mbox.cpp,v 1.5 2008-10-04 14:26:59 dockes Exp
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
 #ifndef TEST_MH_MBOX
+#include "autoconfig.h"
+
 #include <stdio.h>
 #include <fcntl.h>
 #include <errno.h>
@@ -494,6 +496,7 @@ bool MimeHandlerMbox::next_document()
 #include <string>
 using namespace std;
 
+#include "rclconfig.h"
 #include "rclinit.h"
 #include "mh_mbox.h"
 
@@ -508,6 +511,11 @@ Usage(void)
     fprintf(stderr, "%s: usage:\n%s", thisprog, usage);
     exit(1);
 }
+static RclConfig *config;
+RclConfig *RclConfig::getMainConfig()
+{
+    return config;
+}
 
 int main(int argc, char **argv)
 {
@@ -530,8 +538,8 @@ int main(int argc, char **argv)
     Usage();
   string filename = *argv++;argc--;
   string reason;
-  RclConfig *conf = recollinit(RclInitFlags(0), 0, 0, reason, 0);
-  if (conf == 0) {
+  config = recollinit(RclInitFlags(0), 0, 0, reason, 0);
+  if (config == 0) {
       cerr << "init failed " << reason << endl;
       exit(1);
   }
