@@ -74,7 +74,7 @@ void rwSettings(bool writing)
     SETTING_RW(prefs.mainheight, "/Recoll/geometry/height", Num, 0);
     SETTING_RW(prefs.pvwidth, "/Recoll/geometry/pvwidth", Num, 0);
     SETTING_RW(prefs.pvheight, "/Recoll/geometry/pvheight", Num, 0);
-    SETTING_RW(prefs.ssearchTyp, "/Recoll/prefs/simpleSearchTyp", Num, 1);
+    SETTING_RW(prefs.ssearchTyp, "/Recoll/prefs/simpleSearchTyp", Num, 3);
     SETTING_RW(prefs.startWithAdvSearchOpen, 
 	       "/Recoll/prefs/startWithAdvSearchOpen", Bool, false);
     SETTING_RW(prefs.startWithSortToolOpen, 
@@ -144,6 +144,9 @@ void rwSettings(bool writing)
 	       Num, 250);
     SETTING_RW(prefs.syntAbsCtx, "/Recoll/prefs/query/syntAbsCtx", 
 	       Num, 4);
+    SETTING_RW(prefs.autoSuffs, "/Recoll/prefs/query/autoSuffs", , "");
+    SETTING_RW(prefs.autoSuffsEnable, 
+	       "/Recoll/prefs/query/autoSuffsEnable", Bool, false);
 
     SETTING_RW(prefs.sortDepth, "/Recoll/prefs/query/sortDepth",
 	       Num, 100);
@@ -205,9 +208,9 @@ void rwSettings(bool writing)
 	prefs.allExtraDbs = g_dynconf->getStringList(allEdbsSk);
 	const char *cp;
 	if ((cp = getenv("RECOLL_EXTRA_DBS")) != 0) {
-	    list<string> dbl;
+	    vector<string> dbl;
 	    stringToTokens(cp, dbl, ":");
-	    for (list<string>::iterator dit = dbl.begin(); dit != dbl.end();
+	    for (vector<string>::iterator dit = dbl.begin(); dit != dbl.end();
 		 dit++) {
 		string dbdir = path_canon(*dit);
 		path_catslash(dbdir);
@@ -255,5 +258,4 @@ void rwSettings(bool writing)
 	for (list<string>::iterator it = tl.begin(); it != tl.end(); it++)
 	    prefs.asearchSubdirHist.push_front(QString::fromUtf8(it->c_str()));
     }
-
 }
