@@ -732,6 +732,7 @@ bool Preview::makeDocCurrent(const Rcl::Doc& doc, int docnum, bool sametab)
 	closeCurrentTab();
 	return false;
     }
+    raise();
     return true;
 }
 
@@ -762,7 +763,7 @@ class LoadThread : public QThread {
     Rcl::Doc& out;
     const Rcl::Doc& idoc;
     string filename;
-    string tmpdir;
+    TempDir tmpdir;
     int loglevel;
  public: 
     string missing;
@@ -772,18 +773,14 @@ class LoadThread : public QThread {
 	    loglevel = DebugLog::getdbl()->getlevel();
 	}
     ~LoadThread() {
-	if (tmpdir.length()) {
-	    wipedir(tmpdir);
-	    rmdir(tmpdir.c_str());
-	}
     }
     virtual void run() {
 	DebugLog::getdbl()->setloglevel(loglevel);
 	string reason;
-	if (!maketmpdir(tmpdir, reason)) {
+	if (!tmpdir.ok()) {
 	    QMessageBox::critical(0, "Recoll",
 				  Preview::tr("Cannot create temporary directory"));
-	    LOGERR(("Preview: %s\n", reason.c_str()));
+	    LOGERR(("Preview: %s\n", tmpdir.getreason().c_str()));
 	    *statusp = -1;
 	    return;
 	}
@@ -953,6 +950,7 @@ bool Preview::loadDocInCurrentTab(const Rcl::Doc &idoc, int docnum)
     PreviewTextEdit *editor = currentEditor();
     editor->setText("");
     editor->setTextFormat(Qt::RichText);
+    editor->m_data.format = Qt::RichText;
     bool inputishtml = !fdoc.mimetype.compare("text/html");
 
 #if 0
@@ -1030,6 +1028,7 @@ bool Preview::loadDocInCurrentTab(const Rcl::Doc &idoc, int docnum)
 	    qrichlst.push_back(qr);
 	} else {
             editor->setTextFormat(Qt::PlainText);
+            editor->m_data.format = Qt::PlainText;
 	    for (int pos = 0; pos < (int)qr.length(); pos += l) {
 		l = MIN(CHUNKL, qr.length() - pos);
 		qrichlst.push_back(qr.mid(pos, l));
@@ -1126,6 +1125,7 @@ void PreviewTextEdit::toggleFields()
 
     // If currently displaying fields, switch to body text
     if (m_dspflds) {
+        setTextFormat(m_data.format);
 	setText(m_data.richtxt);
         m_dspflds = false;
 	return;
@@ -1133,6 +1133,7 @@ void PreviewTextEdit::toggleFields()
 
     // Else display fields
     m_dspflds = true;
+    setTextFormat(Qt::RichText);
     QString txt = "<html><head></head><body>\n";
     txt += "<b>" + QString::fromLocal8Bit(m_data.url.c_str());
     if (!m_data.ipath.empty())
@@ -1142,7 +1143,8 @@ void PreviewTextEdit::toggleFields()
     for (map<string,string>::const_iterator it = m_data.fdoc.meta.begin();
 	 it != m_data.fdoc.meta.end(); it++) {
 	txt += "<dt>" + QString::fromUtf8(it->first.c_str()) + "</dt> " 
-	    + "<dd>" + QString::fromUtf8(it->second.c_str()) + "</dd>\n";
+	    + "<dd>" + QString::fromUtf8(escapeHtml(it->second).c_str()) 
+            + "</dd>\n";
     }
     txt += "</dl></body></html>";
     setText(txt);
