@@ -33,18 +33,20 @@ using std::pair;
 
 #include <qapplication.h>
 #include <qmessagebox.h>
+#include <qfiledialog.h>
 
 #if (QT_VERSION < 0x040000)
 #include <qcstring.h>
 #include <qpopupmenu.h>
 #include <qradiobutton.h>
 #include <qbuttongroup.h>
-#include <qfiledialog.h>
+#include <qaccel.h>
+#undef RCLQT4
+#define RCLQT3 1
 #else
-#include <q3filedialog.h>
-#define QFileDialog  Q3FileDialog
-#include <q3toolbar.h>
-#define QToolBar  Q3ToolBar
+#undef RCLQT3
+#define RCLQT4 1
+#include <qshortcut.h>
 #endif
 
 #include <qtabwidget.h>
@@ -63,7 +65,6 @@ using std::pair;
 #include <qiconset.h>
 #include <qapplication.h>
 #include <qcursor.h>
-
 #include "recoll.h"
 #include "debuglog.h"
 #include "mimehandler.h"
@@ -92,7 +93,7 @@ extern "C" int XFlush(void *);
 QString g_stringAllStem, g_stringNoStem;
 
 // Taken from qt designer. Don't know why it's needed.
-#if (QT_VERSION < 0x040000)
+#if RCLQT3
 static QIconSet createIconSet(const QString &name)
 {
     QIconSet ic(QPixmap::fromMimeSource(name));
@@ -172,18 +173,45 @@ void RclMain::init()
     }
     preferencesMenu->setItemChecked(curid, true);
 
+    // A shortcut to get the focus back to the search entry. 
+    QKeySequence seq("Ctrl+Shift+s");
+#if RCLQT4
+    QShortcut *sc = new QShortcut(seq, this);
+    connect(sc, SIGNAL (activated()), this, SLOT (focusToSearch()));
+#else
+    QAccel *sc = new QAccel(this);
+    sc->insertItem(seq);
+    connect(sc, SIGNAL (activated(int)), this, SLOT (focusToSearch()));
+#endif
+
+
     // Toolbar+combobox version of the category selector
     QComboBox *catgCMB = 0;
     if (prefs.catgToolBar) {
-	catgCMB = new QComboBox(FALSE, new QToolBar(this), "catCMB");
+        QToolBar *catgToolBar = new QToolBar(this);
+	catgCMB = new QComboBox(FALSE, catgToolBar, "catCMB");
 	catgCMB->insertItem(tr("All"));
-#if (QT_VERSION >= 0x040000)
+#if RCLQT4
+        catgToolBar->setObjectName(QString::fromUtf8("catgToolBar"));
 	catgCMB->setToolTip(tr("Document category filter"));
+        catgToolBar->addWidget(catgCMB);
+        this->addToolBar(Qt::TopToolBarArea, catgToolBar);
 #endif
     }
 
     // Document categories buttons
+#if RCLQT3
     catgBGRP->setColumnLayout(1, Qt::Vertical);
+    connect(catgBGRP, SIGNAL(clicked(int)), this, SLOT(catgFilter(int)));
+#else
+    QHBoxLayout *bgrphbox = new QHBoxLayout(catgBGRP);
+    QButtonGroup *bgrp  = new QButtonGroup(catgBGRP);
+    bgrphbox->addWidget(allRDB);
+    int bgrpid = 0;
+    bgrp->addButton(allRDB, bgrpid++);
+    connect(bgrp, SIGNAL(buttonClicked(int)), this, SLOT(catgFilter(int)));
+#endif
+    allRDB->setChecked(true);
     list<string> cats;
     rclconfig->getMimeCategories(cats);
     // Text for button 0 is not used. Next statement just avoids unused
@@ -197,14 +225,23 @@ void RclMain::init()
 	but->setText(tr(catgnm));
 	if (prefs.catgToolBar && catgCMB)
 	    catgCMB->insertItem(tr(catgnm));
+#if RCLQT4
+        bgrphbox->addWidget(but);
+        bgrp->addButton(but, bgrpid++);
+#endif
     }
+#if RCLQT3
     catgBGRP->setButton(0);
+#else
+    catgBGRP->setLayout(bgrphbox);
+#endif
+
     if (prefs.catgToolBar)
 	catgBGRP->hide();
     // Connections
     connect(sSearch, SIGNAL(startSearch(RefCntr<Rcl::SearchData>)), 
 		this, SLOT(startSearch(RefCntr<Rcl::SearchData>)));
-#if QT_VERSION >= 0x040000
+#if RCLQT4
     sSearch->queryText->installEventFilter(this);
 #else
     sSearch->queryText->lineEdit()->installEventFilter(this);
@@ -219,9 +256,9 @@ void RclMain::init()
     connect(firstPageAction, SIGNAL(activated()), 
 	    resList, SLOT(resultPageFirst()));
     connect(prevPageAction, SIGNAL(activated()), 
-	    resList, SLOT(resultPageBack()));
+	    resList, SLOT(resPageUpOrBack()));
     connect(nextPageAction, SIGNAL(activated()),
-	    resList, SLOT(resultPageNext()));
+	    resList, SLOT(resPageDownOrNext()));
 
     connect(resList, SIGNAL(docExpand(int)), this, SLOT(docExpand(int)));
     connect(resList, SIGNAL(wordSelect(QString)),
@@ -264,15 +301,16 @@ void RclMain::init()
     connect(indexConfigAction, SIGNAL(activated()), this, SLOT(showIndexConfig()));
     connect(queryPrefsAction, SIGNAL(activated()), this, SLOT(showUIPrefs()));
     connect(extIdxAction, SIGNAL(activated()), this, SLOT(showExtIdxDialog()));
-    connect(catgBGRP, SIGNAL(clicked(int)), this, SLOT(catgFilter(int)));
     if (prefs.catgToolBar && catgCMB)
 	connect(catgCMB, SIGNAL(activated(int)), this, SLOT(catgFilter(int)));
+    connect(toggleFullScreenAction, SIGNAL(activated()), 
+            this, SLOT(toggleFullScreen()));
     connect(periodictimer, SIGNAL(timeout()), this, SLOT(periodic100()));
     // Start timer on a slow period (used for checking ^C). Will be
     // speeded up during indexing
     periodictimer->start(1000);
 
-#if (QT_VERSION < 0x040000)
+#if RCLQT3
     nextPageAction->setIconSet(createIconSet("nextpage.png"));
     prevPageAction->setIconSet(createIconSet("prevpage.png"));
     firstPageAction->setIconSet(createIconSet("firstpage.png"));
@@ -322,7 +360,7 @@ void RclMain::initDbOpen()
 		 QString::fromLocal8Bit(rclconfig->getDbDir().c_str()) +
 		 qApp->translate("Main", 
 			       ".\n"
-			       "Click Cancel if you want to edit the configuration file before indexation starts, or Ok to let it proceed."),
+			       "Click Cancel if you want to edit the configuration file before indexing starts, or Ok to let it proceed."),
 		 "Ok", "Cancel", 0,   0)) {
 
 	case 0: // Ok: indexing is going to start.
@@ -350,6 +388,16 @@ void RclMain::initDbOpen()
     }
 }
 
+void RclMain::focusToSearch()
+{
+    LOGDEB(("Giving focus to sSearch\n"));
+    sSearch->queryText->setFocus(
+#if RCLQT4
+        Qt::ShortcutFocusReason
+#endif
+        );
+}
+
 void RclMain::setStemLang(int id)
 {
     LOGDEB(("RclMain::setStemLang(%d)\n", id));
@@ -435,8 +483,11 @@ void RclMain::fileExit()
 {
     LOGDEB(("RclMain: fileExit\n"));
     m_tempfiles.clear();
-    prefs.mainwidth = width();
-    prefs.mainheight = height();
+    // Don't save geometry if we're currently fullscreened
+    if (!isFullScreen()) {
+        prefs.mainwidth = width();
+        prefs.mainheight = height();
+    }
     prefs.ssearchTyp = sSearch->searchTypCMB->currentItem();
     if (asearchform)
 	delete asearchform;
@@ -896,7 +947,7 @@ static bool lookForHtmlBrowser(string &exefile)
 {
     static const char *htmlbrowserlist = 
 	"opera konqueror firefox mozilla netscape epiphany";
-    list<string> blist;
+    vector<string> blist;
     stringToTokens(htmlbrowserlist, blist, " ");
 
     const char *path = getenv("PATH");
@@ -904,7 +955,7 @@ static bool lookForHtmlBrowser(string &exefile)
 	path = "/bin:/usr/bin:/usr/bin/X11:/usr/X11R6/bin:/usr/local/bin";
 
     // Look for each browser 
-    for (list<string>::const_iterator bit = blist.begin(); 
+    for (vector<string>::const_iterator bit = blist.begin(); 
 	 bit != blist.end(); bit++) {
 	if (ExecCmd::which(*bit, exefile, path)) 
 	    return true;
@@ -928,8 +979,14 @@ void RclMain::startNativeViewer(int docnum)
 static string fileurltolocalpath(string url)
 {
     if (url.find("file://") == 0)
-        return url.substr(7, string::npos);
-    return string();
+        url = url.substr(7, string::npos);
+    else
+        return string();
+    string::size_type pos;
+    if ((pos = url.find_last_of("#")) != string::npos) {
+        url.erase(pos);
+    }
+    return url;
 }
 
 void RclMain::startNativeViewer(Rcl::Doc doc)
@@ -1045,6 +1102,32 @@ void RclMain::startNativeViewer(Rcl::Doc doc)
 	url = string("file://") + fn;
     }
 
+    // If using an actual file, check that it exists, and if it is
+    // compressed, we may need an uncompressed version
+    if (!fn.empty() && rclconfig->mimeViewerNeedsUncomp(doc.mimetype)) {
+        if (access(fn.c_str(), R_OK) != 0) {
+            QMessageBox::warning(0, "Recoll", 
+                                 tr("Can't access file: ") + 
+                                 QString::fromLocal8Bit(fn.c_str()));
+            return;
+        }
+        TempFile temp;
+        if (FileInterner::isCompressed(fn, rclconfig)) {
+            if (!FileInterner::maybeUncompressToTemp(temp, fn, rclconfig,  
+                                                     doc)) {
+                QMessageBox::warning(0, "Recoll", 
+                                     tr("Can't uncompress file: ") + 
+                                     QString::fromLocal8Bit(fn.c_str()));
+                return;
+            }
+        }
+        if (!temp.isNull()) {
+            m_tempfiles.push_back(temp);
+            fn = temp->filename();
+            url = string("file://") + fn;
+        }
+    }
+
     // Substitute %xx inside prototype command
     string efftime;
     if (!doc.dmtime.empty() || !doc.fmtime.empty()) {
@@ -1083,6 +1166,7 @@ void RclMain::startNativeViewer(Rcl::Doc doc)
 
     if (!istempfile)
 	historyEnterDoc(g_dynconf, doc.meta[Rcl::Doc::keyudi]);
+    
     // We should actually monitor these processes so that we can
     // delete the temp files when they exit
     LOGDEB(("Executing: [%s]\n", ncmd.c_str()));
@@ -1222,21 +1306,28 @@ void RclMain::catgFilter(int id)
     resList->setDocSource();
 }
 
+void RclMain::toggleFullScreen()
+{
+    if (isFullScreen())
+        showNormal();
+    else
+        showFullScreen();
+}
+
 bool RclMain::eventFilter(QObject *, QEvent *event)
 {
     if (event->type() == QEvent::KeyPress)  {
         LOGDEB2(("RclMain::eventFilter: keypress\n"));
 	QKeyEvent *ke = (QKeyEvent *)event;
-	if (ke->key() == Qt::Key_PageDown || ke->key() == Qt::Key_PageUp ) {
-            // Page up down are sent to the resList where they make sense
-            QApplication::sendEvent(resList, event);
-            return true;
-        } else if (ke->key() == Qt::Key_Home &&
-                   (ke->state() & Qt::ShiftButton)) {
+        // Shift-Home is the shortcut for the 1st result page action, but it is
+        // filtered by the search entry to mean "select all line". We prefer to
+        // keep it for the action as it's easy to find another combination to
+        // select all (ie: home, then shift-end)
+        if (ke->key() == Qt::Key_Home && (ke->state() & Qt::ShiftButton)) {
             // Shift-Home -> first page of results
             resList->resultPageFirst();
             return true;
-        }
+        } 
     }
     return false;
 }
