@@ -34,13 +34,8 @@
 #if QT_VERSION < 0x040000
 #include "rclmain.h"
 #else
-#if QT_VERSION >= 0x040601
-// 4.6.1 uic is broken
-#include "ui_rclmain.h-4.5"
-#else
 #include "ui_rclmain.h"
 #endif
-#endif
 
 //MOC_SKIP_BEGIN
 #if QT_VERSION < 0x040000
@@ -50,10 +45,10 @@ class DummyRclMainBase : public RclMainBase
 };
 #define RCLMAINPARENT QWidget
 #else
-class DummyRclMainBase : public Q3MainWindow, public Ui::RclMainBase
+class DummyRclMainBase : public QMainWindow, public Ui::RclMainBase
 {
-public: DummyRclMainBase(QWidget *parent) :Q3MainWindow(parent){setupUi(this);}
-#define RCLMAINPARENT Q3MainWindow
+public: DummyRclMainBase(QWidget *parent) :QMainWindow(parent){setupUi(this);}
+#define RCLMAINPARENT QMainWindow
 };
 #endif
 //MOC_SKIP_END
@@ -116,6 +111,8 @@ public slots:
     virtual void adjustPrefsMenu();
     virtual void catgFilter(int);
     virtual void initDbOpen();
+    virtual void toggleFullScreen();
+    virtual void focusToSearch();
 
 signals:
     void stemLangChanged(const QString& lang);
