@@ -101,6 +101,11 @@ ResList::ResList(QWidget* parent, const char* name)
     // signals and slots connections
     connect(this, SIGNAL(linkClicked(const QString &, int)), 
 	    this, SLOT(linkWasClicked(const QString &, int)));
+#if 0
+    // See comments in "highlighted
+    connect(this, SIGNAL(highlighted(const QString &)), 
+	    this, SLOT(highlighted(const QString &)));
+#endif
     connect(this, SIGNAL(headerClicked()), this, SLOT(showQueryDetails()));
     connect(this, SIGNAL(doubleClicked(int,int)), 
 	    this, SLOT(doubleClicked(int, int)));
@@ -179,6 +184,8 @@ void ResList::setDocSource()
 							    m_sortspecs,
 							    title));
     }
+    // Reset the page size in case the preference was changed
+    m_pager->setPageSize(prefs.respagesize);
     m_pager->setDocSource(m_docSource);
     resultPageNext();
 }
@@ -343,6 +350,13 @@ int ResList::getResCnt()
     return m_docSource->getResCnt();
 }
 
+void ResList::highlighted(const QString& )
+{
+    // This is supposedly called when a link is preactivated (hover or tab
+    // traversal, but is not actually called for tabs. We would have liked to
+    // give some kind of visual feedback for tab traversal
+}
+
 
 #if 1 || (QT_VERSION < 0x040000)
 #define SCROLLYPOS contentsY()
@@ -382,6 +396,8 @@ void ResList::resultPageBack()
 // Go to the first page
 void ResList::resultPageFirst()
 {
+    // In case the preference was changed
+    m_pager->setPageSize(prefs.respagesize);
     m_pager->resultPageFirst();
     displayPage();
 }
