@@ -24,8 +24,10 @@ using std::string;
 #include "rcldb.h"
 #include "searchdata.h"
 
-extern Rcl::SearchData *wasaStringToRcl(const string& query, string &reason);
+extern Rcl::SearchData *wasaStringToRcl(const string& query, string &reason,
+                                        const string& autosuffs = string());
 class WasaQuery;
-extern Rcl::SearchData *wasaQueryToRcl(WasaQuery *wasa);
+extern Rcl::SearchData *wasaQueryToRcl(WasaQuery *wasa,
+                                       const string& autosuffs = string());
 
 #endif /* _WASATORCL_H_INCLUDED_ */
