@@ -33,7 +33,7 @@ static char rcsid[] = "@(#$Id: rcldb.cpp,v 1.154 2008-12-17 16:19:58 dockes Exp
 using namespace std;
 #endif /* NO_NAMESPACES */
 
-#include "xapian/version.h"
+#include "xapian.h"
 
 #include "rclconfig.h"
 #include "rcldb.h"
@@ -556,6 +556,11 @@ bool Db::open(OpenMode mode)
 		int action = (mode == DbUpd) ? Xapian::DB_CREATE_OR_OPEN :
 		    Xapian::DB_CREATE_OR_OVERWRITE;
 		m_ndb->xwdb = Xapian::WritableDatabase(dir, action);
+                // If db is empty, write the data format version at once
+                // to avoid stupid error messages:
+                if (m_ndb->xwdb.get_doccount() == 0)
+                    m_ndb->xwdb.set_metadata(RCL_IDX_VERSION_KEY, 
+                                             RCL_IDX_VERSION);
 		m_ndb->m_iswritable = true;
 		// We open a readonly object in all cases (possibly in
 		// addition to the r/w one) because some operations
@@ -577,19 +582,13 @@ bool Db::open(OpenMode mode)
 	    m_ndb->xrdb = Xapian::Database(dir);
 	    for (list<string>::iterator it = m_extraDbs.begin();
 		 it != m_extraDbs.end(); it++) {
-		string aerr;
 		LOGDEB(("Db::Open: adding query db [%s]\n", it->c_str()));
-		aerr.erase();
-		try {
-		    // Make this non-fatal
-		    m_ndb->xrdb.add_database(Xapian::Database(*it));
-		} XCATCHERROR(aerr);
-		if (!aerr.empty())
-		    LOGERR(("Db::Open: error while trying to add database "
-			    "from [%s]: %s\n", it->c_str(), aerr.c_str()));
+                // Used to be non-fatal (1.13 and older) but I can't see why
+                m_ndb->xrdb.add_database(Xapian::Database(*it));
 	    }
 	    break;
 	}
+
 	// Check index format version. Must not try to check a just created or
 	// truncated db
 	if (mode != DbTrunc && m_ndb->xdb().get_doccount() > 0) {
@@ -607,6 +606,8 @@ bool Db::open(OpenMode mode)
 	m_basedir = dir;
 	return true;
     } XCATCHERROR(ermsg);
+
+    m_reason = ermsg;
     LOGERR(("Db::open: exception while opening [%s]: %s\n", 
 	    dir.c_str(), ermsg.c_str()));
     return false;
@@ -766,6 +767,10 @@ bool Db::fieldToPrefix(const string& fld, string &pfx)
 	fldToPrefs["keyword"] = "K";
 	fldToPrefs["tag"] = "K";
 	fldToPrefs["tags"] = "K";
+
+        fldToPrefs["xapyear"] = "Y";
+        fldToPrefs["xapyearmon"] = "M";
+        fldToPrefs["xapdate"] = "D";
     }
 
     if (m_config && m_config->getFieldPrefix(fld, pfx))
@@ -781,16 +786,15 @@ bool Db::fieldToPrefix(const string& fld, string &pfx)
 }
 
 
-// The text splitter callback class which receives words from the
-// splitter and adds postings to the Xapian document.
-class mySplitterCB : public TextSplitCB {
+// The splitter breaks text into words and adds postings to the Xapian document.
+class TextSplitDb : public TextSplit {
  public:
     Xapian::Document &doc;   // Xapian document 
     Xapian::termpos basepos; // Base for document section
     Xapian::termpos curpos;  // Current position. Used to set basepos for the
                              // following section
     StopList &stops;
-    mySplitterCB(Xapian::Document &d, StopList &_stops) 
+    TextSplitDb(Xapian::Document &d, StopList &_stops) 
 	: doc(d), basepos(1), curpos(0), stops(_stops)
     {}
     bool takeword(const std::string &term, int pos, int, int);
@@ -802,15 +806,16 @@ private:
     string  prefix; 
 };
 
-// Callback for the document to word splitting class during indexation
-bool mySplitterCB::takeword(const std::string &_term, int pos, int, int)
+// Get one term from the doc, remove accents and lowercase, then add posting
+bool TextSplitDb::takeword(const std::string &_term, int pos, int, int)
 {
 #if 0
-    LOGDEB(("mySplitterCB::takeword:splitCb: [%s]\n", _term.c_str()));
+    LOGDEB(("TextSplitDb::takeword: [%s]\n", _term.c_str()));
 #endif
     string term;
     if (!unacmaybefold(_term, term, "UTF-8", true)) {
-	LOGINFO(("Db::splitter::takeword: unac failed for [%s]\n", _term.c_str()));
+	LOGINFO(("Db::splitter::takeword: unac failed for [%s]\n", 
+                 _term.c_str()));
 	term.clear();
 	// We don't generate a fatal error because of a bad term
 	return true;
@@ -892,14 +897,13 @@ bool Db::addOrUpdate(const string &udi, const string &parent_udi,
     Doc doc = idoc;
 
     Xapian::Document newdocument;
-    mySplitterCB splitData(newdocument, m_stops);
-    TextSplit splitter(&splitData);
+    TextSplitDb splitter(newdocument, m_stops);
 
     // Split and index file name as document term(s)
     LOGDEB2(("Db::add: split file name [%s]\n", fn.c_str()));
     if (!splitter.text_to_words(doc.utf8fn))
         LOGDEB(("Db::addOrUpdate: split failed for file name\n"));
-    splitData.basepos += splitData.curpos + 100;
+    splitter.basepos += splitter.curpos + 100;
 
     // Index textual metadata.  These are all indexed as text with
     // positions, as we may want to do phrase searches with them (this
@@ -919,19 +923,19 @@ bool Db::addOrUpdate(const string &udi, const string &parent_udi,
 	    LOGDEB0(("Db::add: field [%s] pfx [%s]: [%s]\n", 
 		    meta_it->first.c_str(), pfx.c_str(), 
 		    meta_it->second.c_str()));
-	    splitData.setprefix(pfx); // Subject
+	    splitter.setprefix(pfx); // Subject
 	    if (!splitter.text_to_words(meta_it->second))
                 LOGDEB(("Db::addOrUpdate: split failed for %s\n", 
                         meta_it->first.c_str()));
-	    splitData.setprefix(string());
-	    splitData.basepos += splitData.curpos + 100;
+	    splitter.setprefix(string());
+	    splitter.basepos += splitter.curpos + 100;
 	}
     }
 
-    if (splitData.curpos < baseTextPosition)
-	splitData.basepos = baseTextPosition;
+    if (splitter.curpos < baseTextPosition)
+	splitter.basepos = baseTextPosition;
     else
-	splitData.basepos += splitData.curpos + 100;
+	splitter.basepos += splitter.curpos + 100;
 
     // Split and index body text
     LOGDEB2(("Db::add: split body\n"));
@@ -958,6 +962,15 @@ bool Db::addOrUpdate(const string &udi, const string &parent_udi,
 	    fn = string("XSFN") + fn;
 	    newdocument.add_term(fn);
 	}
+        // Store utf8fn inside the metadata array as keyfn
+        // (="filename") so that it can be accessed by the "stored"
+        // processing below, without special-casing it. We only do it
+        // if keyfn is currently empty, because there could be a value
+        // already (ie for a mail attachment with a file name
+        // attribute)
+	if (doc.meta[Doc::keyfn].empty()) {
+            doc.meta[Doc::keyfn] = doc.utf8fn;
+	}
     }
 
     // Udi unique term: this is used for file existence/uptodate
@@ -1341,11 +1354,11 @@ bool Db::filenameWildExp(const string& fnexp, list<string>& names)
     } // else let it be
     LOGDEB(("Rcl::Db::filenameWildExp: pattern: [%s]\n", pattern.c_str()));
 
-    list<TermMatchEntry> entries;
-    if (!termMatch(ET_WILD, string(), pattern, entries, 1000, Doc::keyfn))
+    TermMatchResult result;
+    if (!termMatch(ET_WILD, string(), pattern, result, 1000, Doc::keyfn))
 	return false;
-    for (list<TermMatchEntry>::const_iterator it = entries.begin();
-	 it != entries.end(); it++) 
+    for (list<TermMatchEntry>::const_iterator it = result.entries.begin();
+	 it != result.entries.end(); it++) 
 	names.push_back(it->term);
 
     if (names.empty()) {
@@ -1356,6 +1369,28 @@ bool Db::filenameWildExp(const string& fnexp, list<string>& names)
     return true;
 }
 
+// Walk the Y terms and return min/max
+bool Db::maxYearSpan(int *minyear, int *maxyear)
+{
+    *minyear = 1000000; 
+    *maxyear = -1000000;
+    TermMatchResult result;
+    if (!termMatch(ET_WILD, string(), "*", result, 5000, "xapyear"))
+	return false;
+    for (list<TermMatchEntry>::const_iterator it = result.entries.begin();
+	 it != result.entries.end(); it++) {
+        if (!it->term.empty()) {
+            int year = atoi(it->term.c_str()+1);
+            if (year < *minyear)
+                *minyear = year;
+            if (year > *maxyear)
+                *maxyear = year;
+        }
+    }
+    return true;
+}
+
+
 class TermMatchCmpByWcf {
 public:
     int operator()(const TermMatchEntry& l, const TermMatchEntry& r) {
@@ -1376,18 +1411,17 @@ public:
 };
 
 bool Db::stemExpand(const string &lang, const string &term, 
-		    list<TermMatchEntry>& result, int max)
+		    TermMatchResult& result, int max)
 {
     list<string> dirs = m_extraDbs;
     dirs.push_front(m_basedir);
-    for (list<string>::iterator it = dirs.begin();
-	 it != dirs.end(); it++) {
+    for (list<string>::iterator it = dirs.begin(); it != dirs.end(); it++) {
 	list<string> more;
 	StemDb::stemExpand(*it, lang, term, more);
 	LOGDEB1(("Db::stemExpand: Got %d from %s\n", 
 		 more.size(), it->c_str()));
-	result.insert(result.end(), more.begin(), more.end());
-	if (result.size() >= (unsigned int)max)
+	result.entries.insert(result.entries.end(), more.begin(), more.end());
+	if (result.entries.size() >= (unsigned int)max)
 	    break;
     }
     LOGDEB1(("Db:::stemExpand: final count %d \n", result.size()));
@@ -1413,7 +1447,7 @@ const string regSpecChars = "(.[{";
 // Find all index terms that match a wildcard or regular expression
 bool Db::termMatch(MatchType typ, const string &lang,
 		   const string &root, 
-		   list<TermMatchEntry>& res,
+		   TermMatchResult& res,
 		   int max, 
 		   const string& field,
                    string *prefixp
@@ -1424,6 +1458,10 @@ bool Db::termMatch(MatchType typ, const string &lang,
     Xapian::Database xdb = m_ndb->xdb();
 
     res.clear();
+    XAPTRY(res.dbdoccount = xdb.get_doccount();
+           res.dbavgdoclen = xdb.get_avlength(), xdb, m_reason);
+    if (!m_reason.empty())
+        return false;
 
     // Get rid of capitals and accents
     string droot;
@@ -1447,18 +1485,19 @@ bool Db::termMatch(MatchType typ, const string &lang,
     if (typ == ET_STEM) {
 	if (!stemExpand(lang, root, res, max))
 	    return false;
-	res.sort();
-	res.unique();
-	for (list<TermMatchEntry>::iterator it = res.begin(); 
-	     it != res.end(); it++) {
-	    XAPTRY(it->wcf = xdb.get_collection_freq(it->term),
+	res.entries.sort();
+	res.entries.unique();
+	for (list<TermMatchEntry>::iterator it = res.entries.begin(); 
+	     it != res.entries.end(); it++) {
+	    XAPTRY(it->wcf = xdb.get_collection_freq(it->term);
+                   it->docs = xdb.get_termfreq(it->term),
                    xdb, m_reason);
             if (!m_reason.empty())
                 return false;
 	    LOGDEB1(("termMatch: %d [%s]\n", it->wcf, it->term.c_str()));
 	}
         if (!prefix.empty())
-            addPrefix(res, prefix);
+            addPrefix(res.entries, prefix);
     } else {
 	regex_t reg;
 	int errcode;
@@ -1469,7 +1508,7 @@ bool Db::termMatch(MatchType typ, const string &lang,
 		char errbuf[200];
 		regerror(errcode, &reg, errbuf, 199);
 		LOGERR(("termMatch: regcomp failed: %s\n", errbuf));
-		res.push_back(string(errbuf));
+		res.entries.push_back(string(errbuf));
 		regfree(&reg);
 		return false;
 	    }
@@ -1509,7 +1548,9 @@ bool Db::termMatch(MatchType typ, const string &lang,
                             continue;
                     }
                     // Do we want stem expansion here? We don't do it for now
-                    res.push_back(TermMatchEntry(*it, it.get_termfreq()));
+                    res.entries.push_back(TermMatchEntry(*it, 
+                                                   xdb.get_collection_freq(*it),
+                                                   it.get_termfreq()));
                     ++n;
                 }
                 m_reason.erase();
@@ -1533,13 +1574,13 @@ bool Db::termMatch(MatchType typ, const string &lang,
     }
 
     TermMatchCmpByTerm tcmp;
-    res.sort(tcmp);
+    res.entries.sort(tcmp);
     TermMatchTermEqual teq;
-    res.unique(teq);
+    res.entries.unique(teq);
     TermMatchCmpByWcf wcmp;
-    res.sort(wcmp);
+    res.entries.sort(wcmp);
     if (max > 0) {
-	res.resize(MIN(res.size(), (unsigned int)max));
+	res.entries.resize(MIN(res.entries.size(), (unsigned int)max));
     }
     return true;
 }
