@@ -19,6 +19,7 @@ static char rcsid[] = "@(#$Id: searchdata.cpp,v 1.32 2008-12-19 09:55:36 dockes
  */
 
 // Handle translation from rcl's SearchData structures to Xapian Queries
+#include <stdio.h>
 
 #include <string>
 #include <vector>
@@ -46,33 +47,107 @@ typedef  vector<SearchDataClause *>::const_iterator qlist_cit_t;
 
 static const int original_term_wqf_booster = 10;
 
+/* The dates-to-query routine is is lifted quasi-verbatim but
+ *  modified from xapian-omega:date.cc. Copyright info:
+ *
+ * Copyright 1999,2000,2001 BrightStation PLC
+ * Copyright 2001 James Aylett
+ * Copyright 2001,2002 Ananova Ltd
+ * Copyright 2002 Intercede 1749 Ltd
+ * Copyright 2002,2003,2006 Olly Betts
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; either version 2 of the
+ * License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
+ * USA
+ */
+static Xapian::Query
+date_range_filter(int y1, int m1, int d1, int y2, int m2, int d2)
+{
+    // Xapian uses a smallbuf and snprintf. Can't be bothered, we're
+    // only doing %d's !
+    char buf[200];
+    sprintf(buf, "D%04d%02d", y1, m1);
+    vector<Xapian::Query> v;
+
+    int d_last = monthdays(m1, y1);
+    int d_end = d_last;
+    if (y1 == y2 && m1 == m2 && d2 < d_last) {
+	d_end = d2;
+    }
+    // Deal with any initial partial month
+    if (d1 > 1 || d_end < d_last) {
+    	for ( ; d1 <= d_end ; d1++) {
+	    sprintf(buf + 7, "%02d", d1);
+	    v.push_back(Xapian::Query(buf));
+	}
+    } else {
+	buf[0] = 'M';
+	v.push_back(Xapian::Query(buf));
+    }
+    
+    if (y1 == y2 && m1 == m2) {
+	return Xapian::Query(Xapian::Query::OP_OR, v.begin(), v.end());
+    }
+
+    int m_last = (y1 < y2) ? 12 : m2 - 1;
+    while (++m1 <= m_last) {
+	sprintf(buf + 5, "%02d", m1);
+	buf[0] = 'M';
+	v.push_back(Xapian::Query(buf));
+    }
+	
+    if (y1 < y2) {
+	while (++y1 < y2) {
+	    sprintf(buf + 1, "%04d", y1);
+	    buf[0] = 'Y';
+	    v.push_back(Xapian::Query(buf));
+	}
+	sprintf(buf + 1, "%04d", y2);
+	buf[0] = 'M';
+	for (m1 = 1; m1 < m2; m1++) {
+	    sprintf(buf + 5, "%02d", m1);
+	    v.push_back(Xapian::Query(buf));
+	}
+    }
+	
+    sprintf(buf + 5, "%02d", m2);
+
+    // Deal with any final partial month
+    if (d2 < monthdays(m2, y2)) {
+	buf[0] = 'D';
+    	for (d1 = 1 ; d1 <= d2; d1++) {
+	    sprintf(buf + 7, "%02d", d1);
+	    v.push_back(Xapian::Query(buf));
+	}
+    } else {
+	buf[0] = 'M';
+	v.push_back(Xapian::Query(buf));
+    }
+
+    return Xapian::Query(Xapian::Query::OP_OR, v.begin(), v.end());
+}
+
 bool SearchData::toNativeQuery(Rcl::Db &db, void *d)
 {
     Xapian::Query xq;
     m_reason.erase();
 
-    if (m_query.size() < 1) {
+    if (!m_query.size() && !m_haveDates) {
 	m_reason = "empty query";
 	return false;
     }
 
-    // It's not allowed to have a pure negative query and also it
-    // seems that Xapian doesn't like the first element to be AND_NOT
-    qlist_it_t itnotneg = m_query.end();
-    for (qlist_it_t it = m_query.begin(); it != m_query.end(); it++) {
-	if ((*it)->m_tp != SCLT_EXCL) {
-	    itnotneg = it;
-	    break;
-	}
-    }
-    if (itnotneg == m_query.end()) {
-	LOGERR(("SearchData::toNativeQuery: can't have all negative clauses"));
-	m_reason = "Can't have only negative clauses";
-	return false;
-    }
-    if ((*m_query.begin())->m_tp == SCLT_EXCL) 
-	iter_swap(m_query.begin(), itnotneg);
-
     // Walk the clause list translating each in turn and building the 
     // Xapian query tree
     for (qlist_it_t it = m_query.begin(); it != m_query.end(); it++) {
@@ -82,18 +157,68 @@ bool SearchData::toNativeQuery(Rcl::Db &db, void *d)
 	    m_reason = (*it)->getReason();
 	    return false;
 	}	    
-
+        if (nq.empty()) {
+            LOGDEB(("SearchData::toNativeQuery: skipping empty clause\n"));
+            continue;
+        }
 	// If this structure is an AND list, must use AND_NOT for excl clauses.
 	// Else this is an OR list, and there can't be excl clauses (checked by
 	// addClause())
 	Xapian::Query::op op;
 	if (m_tp == SCLT_AND) {
-	    op = (*it)->m_tp == SCLT_EXCL ? 
-		Xapian::Query::OP_AND_NOT: Xapian::Query::OP_AND;
+            if ((*it)->m_tp == SCLT_EXCL) {
+                op =  Xapian::Query::OP_AND_NOT;
+            } else {
+                op =  Xapian::Query::OP_AND;
+            }
 	} else {
 	    op = Xapian::Query::OP_OR;
 	}
-	xq = xq.empty() ? nq : Xapian::Query(op, xq, nq);
+        if (xq.empty()) {
+            if (op == Xapian::Query::OP_AND_NOT)
+                xq = Xapian::Query(op, Xapian::Query::MatchAll, nq);
+            else 
+                xq = nq;
+        } else {
+            xq = Xapian::Query(op, xq, nq);
+        }
+    }
+        
+    if (m_haveDates) {
+        // If one of the extremities is unset, compute db extremas
+        if (m_dates.y1 == 0 || m_dates.y2 == 0) {
+            int minyear = 1970, maxyear = 2100;
+            if (!db.maxYearSpan(&minyear, &maxyear)) {
+                LOGERR(("Can't retrieve index min/max dates\n"));
+                //whatever, go on.
+            }
+            if (m_dates.y1 == 0) {
+                m_dates.y1 = minyear;
+                m_dates.m1 = 1;
+                m_dates.d1 = 1;
+            }
+            if (m_dates.y2 == 0) {
+                m_dates.y2 = maxyear;
+                m_dates.m2 = 12;
+                m_dates.d2 = 31;
+            }
+        }
+        LOGDEB(("Db::toNativeQuery: date interval: %d-%d-%d/%d-%d-%d\n",
+                m_dates.y1, m_dates.m1, m_dates.d1,
+                m_dates.y2, m_dates.m2, m_dates.d2));
+        Xapian::Query dq = date_range_filter(m_dates.y1, m_dates.m1, m_dates.d1,
+                m_dates.y2, m_dates.m2, m_dates.d2);
+        if (dq.empty()) {
+            LOGINFO(("Db::toNativeQuery: date filter is empty\n"));
+        }
+        // If no probabilistic query is provided then promote the daterange
+        // filter to be THE query instead of filtering an empty query.
+        if (xq.empty()) {
+            LOGINFO(("Db::toNativeQuery: proba query is empty\n"));
+            xq = dq;
+        } else {
+            xq = Xapian::Query(Xapian::Query::OP_FILTER, xq, dq);
+        }
     }
 
     // Add the file type filtering clause if any
@@ -113,7 +238,6 @@ bool SearchData::toNativeQuery(Rcl::Db &db, void *d)
 	    }
 	}
 	    
-	list<Xapian::Query> pqueries;
 	Xapian::Query tq;
 	for (vector<string>::iterator it = exptps.begin(); 
 	     it != exptps.end(); it++) {
@@ -154,6 +278,7 @@ void SearchData::erase() {
     m_topdir.erase();
     m_description.erase();
     m_reason.erase();
+    m_haveDates = false;
 }
 
 // Am I a file name only search ? This is to turn off term highlighting
@@ -188,25 +313,28 @@ void SearchData::getUTerms(vector<string>& terms) const
 // phrases. This is for parts of the user entry which would appear as
 // a single word because there is no white space inside, but are
 // actually multiple terms to rcldb (ie term1,term2)
-class wsQData : public TextSplitCB {
+class TextSplitQ : public TextSplit {
  public:
-    wsQData(const StopList &_stops) 
-	: stops(_stops), alltermcount(0)
+    TextSplitQ(Flags flags, const StopList &_stops) 
+	: TextSplit(flags), stops(_stops), alltermcount(0), lastpos(0)
     {}
-    bool takeword(const std::string &interm, int , int, int) {
+    bool takeword(const std::string &interm, int pos, int, int) {
 	alltermcount++;
-	LOGDEB1(("wsQData::takeword: %s\n", interm.c_str()));
+        lastpos = pos
+	LOGDEB1(("TextSplitQ::takeword: %s\n", interm.c_str()));
 
 	// Check if the first letter is a majuscule in which
 	// case we do not want to do stem expansion. Note that
 	// the test is convoluted and possibly problematic
 	string noacterm, noaclowterm;
 	if (!unacmaybefold(interm, noacterm, "UTF-8", false)) {
-	    LOGINFO(("SearchData::splitter::takeword: unac failed for [%s]\n", interm.c_str()));
+	    LOGINFO(("SearchData::splitter::takeword: unac failed for [%s]\n", 
+                     interm.c_str()));
 	    return true;
 	} 
 	if (!unacmaybefold(noacterm, noaclowterm, "UTF-8", true)) {
-	    LOGINFO(("SearchData::splitter::takeword: unac failed for [%s]\n", noacterm.c_str()));
+	    LOGINFO(("SearchData::splitter::takeword: unac failed for [%s]\n", 
+                     noacterm.c_str()));
 	    return true;
 	}
 	bool nostemexp = false;
@@ -216,7 +344,8 @@ class wsQData : public TextSplitCB {
 	    nostemexp = true;
 
 	if (stops.hasStops() && stops.isStop(noaclowterm)) {
-	    LOGDEB1(("wsQData::takeword [%s] in stop list\n", noaclowterm.c_str()));
+	    LOGDEB1(("TextSplitQ::takeword [%s] in stop list\n", 
+                     noaclowterm.c_str()));
 	    return true;
 	}
 	terms.push_back(noaclowterm);
@@ -230,6 +359,7 @@ class wsQData : public TextSplitCB {
     // Count of terms including stopwords: this is for adjusting
     // phrase/near slack
     int alltermcount; 
+    int lastpos;
 };
 
 // A class used to translate a user compound string (*not* a query
@@ -271,7 +401,7 @@ private:
     // After splitting entry on whitespace: process non-phrase element
     void processSimpleSpan(const string& span, bool nostemexp, list<Xapian::Query> &pqueries);
     // Process phrase/near element
-    void processPhraseOrNear(wsQData *splitData, 
+    void processPhraseOrNear(TextSplitQ *splitData, 
 			     list<Xapian::Query> &pqueries,
 			     bool useNear, int slack);
 
@@ -339,18 +469,18 @@ void StringToXapianQ::expandTerm(bool nostemexp,
         if (prefix)
             *prefix = pfx;
     } else {
-	list<TermMatchEntry> l;
+	TermMatchResult res;
 	if (haswild) {
-	    m_db.termMatch(Rcl::Db::ET_WILD, m_stemlang, term, l, -1, m_field,
-                           prefix);
+	    m_db.termMatch(Rcl::Db::ET_WILD, m_stemlang, term, res, -1, 
+                           m_field, prefix);
 	} else {
 	    sterm = term;
             m_uterms.push_back(sterm);
-	    m_db.termMatch(Rcl::Db::ET_STEM, m_stemlang, term, l, -1, m_field,
+	    m_db.termMatch(Rcl::Db::ET_STEM, m_stemlang, term, res, -1, m_field,
                            prefix);
 	}
-	for (list<TermMatchEntry>::const_iterator it = l.begin(); 
-	     it != l.end(); it++) {
+	for (list<TermMatchEntry>::const_iterator it = res.entries.begin(); 
+	     it != res.entries.end(); it++) {
 	    exp.push_back(it->term);
 	}
     }
@@ -420,7 +550,7 @@ void StringToXapianQ::processSimpleSpan(const string& span, bool nostemexp,
 // NEAR xapian query, the elements of which can themselves be OR
 // queries if the terms get expanded by stemming or wildcards (we
 // don't do stemming for PHRASE though)
-void StringToXapianQ::processPhraseOrNear(wsQData *splitData, 
+void StringToXapianQ::processPhraseOrNear(TextSplitQ *splitData, 
 					  list<Xapian::Query> &pqueries,
 					  bool useNear, int slack)
 {
@@ -453,8 +583,10 @@ void StringToXapianQ::processPhraseOrNear(wsQData *splitData,
 
     // Generate an appropriate PHRASE/NEAR query with adjusted slack
     // For phrases, give a relevance boost like we do for original terms
+    LOGDEB2(("PHRASE/NEAR: alltermcount %d lastpos %d\n", 
+             splitData->alltermcount, splitData->lastpos));
     Xapian::Query xq(op, orqueries.begin(), orqueries.end(),
-		     splitData->alltermcount + slack);
+		     splitData->lastpos + 1 + slack);
     if (op == Xapian::Query::OP_PHRASE)
 	xq = Xapian::Query(Xapian::Query::OP_SCALE_WEIGHT, xq, 
 			   original_term_wqf_booster);
@@ -527,31 +659,31 @@ bool StringToXapianQ::processUserString(const string &iq,
 	    // We now adjust the phrase/near slack by the term count
 	    // difference (this is mainly better for cjk where this is a very
 	    // common occurrence because of the ngrams thing.
-	    wsQData splitDataS(stops), splitDataW(stops);
-	    TextSplit splitterS(&splitDataS, 
-				TextSplit::Flags(TextSplit::TXTS_ONLYSPANS | 
-						 TextSplit::TXTS_KEEPWILD));
+	    TextSplitQ splitterS(TextSplit::Flags(TextSplit::TXTS_ONLYSPANS | 
+                                                  TextSplit::TXTS_KEEPWILD), 
+                                 stops);
 	    splitterS.text_to_words(*it);
-	    TextSplit splitterW(&splitDataW, 
-				TextSplit::Flags(TextSplit::TXTS_NOSPANS | 
-						 TextSplit::TXTS_KEEPWILD));
+	    TextSplitQ splitterW(TextSplit::Flags(TextSplit::TXTS_NOSPANS | 
+                                                  TextSplit::TXTS_KEEPWILD),
+                                 stops);
 	    splitterW.text_to_words(*it);
-	    wsQData *splitData = &splitDataS;
-	    if (splitDataS.terms.size() > 1 && 
-		splitDataS.terms.size() != splitDataW.terms.size()) {
-		slack += splitDataW.terms.size() - splitDataS.terms.size();
+	    TextSplitQ *splitter = &splitterS;
+	    if (splitterS.terms.size() > 1 && 
+		splitterS.terms.size() != splitterW.terms.size()) {
+		slack += splitterW.terms.size() - splitterS.terms.size();
 		// used to: splitData = &splitDataW;
 	    }
 
-	    LOGDEB0(("strToXapianQ: termcount: %d\n", splitData->terms.size()));
-	    switch (splitData->terms.size()) {
+	    LOGDEB0(("strToXapianQ: termcount: %d\n", splitter->terms.size()));
+	    switch (splitter->terms.size()) {
 	    case 0: 
 		continue;// ??
 	    case 1: 
-		processSimpleSpan(splitData->terms.front(), splitData->nostemexps.front(), pqueries);
+		processSimpleSpan(splitter->terms.front(), 
+                                  splitter->nostemexps.front(), pqueries);
 		break;
 	    default:
-		processPhraseOrNear(splitData, pqueries, useNear, slack);
+		processPhraseOrNear(splitter, pqueries, useNear, slack);
 	    }
 	}
     } catch (const Xapian::Error &e) {
@@ -608,7 +740,7 @@ bool SearchDataClauseSimple::toNativeQuery(Rcl::Db &db, void *p,
 	return false;
     if (pqueries.empty()) {
 	LOGERR(("SearchDataClauseSimple: resolved to null query\n"));
-	return false;
+	return true;
     }
     tr.getTerms(m_terms, m_groups);
     tr.getUTerms(m_uterms);
