@@ -95,11 +95,12 @@ public:
     enum StatusCode {STATUS_ERROR=0, STATUS_RO=1, STATUS_RW=2};
     virtual ~ConfNull() {};
     virtual int get(const string &name, string &value, 
-		    const string &sk = string()) = 0;
+		    const string &sk = string()) const = 0;
+    virtual bool hasNameAnywhere(const string& nm) = 0;
     virtual int set(const string &nm, const string &val, 
 		    const string &sk = string()) = 0;
-    virtual bool ok() = 0;
-    virtual list<string> getNames(const string &sk) = 0;
+    virtual bool ok() const = 0;
+    virtual list<string> getNames(const string &sk, const char* = 0) = 0;
     virtual int erase(const string &, const string &) = 0;
     virtual int eraseKey(const string &) = 0;
     virtual void listall()  {};
@@ -127,7 +128,7 @@ public:
      * @param readonly if true open readonly, else rw
      * @param tildexp  try tilde (home dir) expansion for subsection names
      */
-    ConfSimple(string *data, int readonly = 0, bool tildexp = false);
+    ConfSimple(const string& data, int readonly = 0, bool tildexp = false);
 
     /**
      * Build an empty object. This will be memory only, with no backing store.
@@ -156,7 +157,8 @@ public:
      * global space if sk is empty).
      * @return 0 if name not found, 1 else
      */
-    virtual int get(const string &name, string &value, const string &sk = string());
+    virtual int get(const string &name, string &value, 
+                    const string &sk = string()) const;
 
     /** 
      * Set value for named parameter in specified subsection (or global)
@@ -174,8 +176,8 @@ public:
      */
     virtual int eraseKey(const string &sk);
 
-    virtual StatusCode getStatus(); 
-    virtual bool ok() {return getStatus() != STATUS_ERROR;}
+    virtual StatusCode getStatus() const;
+    virtual bool ok() const {return getStatus() != STATUS_ERROR;}
 
     /** 
      * Walk the configuration values, calling function for each.
@@ -193,10 +195,12 @@ public:
     /** List all values to stdout */
     virtual void listall();
 
-    /**
-     * Return all names in given submap
-     */
-    virtual list<string> getNames(const string &sk);
+    /** Return all names in given submap. */
+    virtual list<string> getNames(const string &sk, const char *pattern = 0);
+
+    /** Check if name is present in any submap. This is relatively expensive
+     * but useful for saving further processing sometimes */
+    virtual bool hasNameAnywhere(const string& nm);
 
     /**
      * Return all subkeys 
@@ -209,13 +213,11 @@ public:
      * Copy constructor. Expensive but less so than a full rebuild
      */
     ConfSimple(const ConfSimple &rhs) 
-	: ConfNull(), m_data(0) 
+	: ConfNull()
     {
 	if ((status = rhs.status) == STATUS_ERROR)
 	    return;
 	m_filename = rhs.m_filename;
-	// Note: we just share the pointer, this doesnt belong to us
-	m_data = rhs.m_data;
 	m_submaps = rhs.m_submaps;
     }
 
@@ -226,21 +228,22 @@ public:
     {
 	if (this != &rhs && (status = rhs.status) != STATUS_ERROR) {
 	    m_filename = rhs.m_filename;
-	    // Note: we don't own data. Just share the pointer
-	    m_data = rhs.m_data;
 	    m_submaps = rhs.m_submaps;
 	}
 	return *this;
     }
 
+    /**
+     * Write in file format to out
+     */
+    bool write(ostream& out) const;
+
 protected:
     bool dotildexpand;
     StatusCode status;
 private:
     // Set if we're working with a file
     string                            m_filename; 
-    // Set if we're working with an in-memory string
-    string                           *m_data;
     // Configuration data submaps (one per subkey, the main data has a
     // null subkey)
     map<string, map<string, string> > m_submaps;
@@ -253,7 +256,6 @@ private:
 
     void parseinput(istream& input);
     bool write();
-    bool write(ostream& out);
     // Internal version of set: no RW checking
     virtual int i_set(const string &nm, const string &val, 
 		      const string &sk, bool init = false);
@@ -272,6 +274,9 @@ private:
  * NOTE: contrary to common behaviour, the global or root space is NOT
  * designated by '/' but by '' (empty subkey). A '/' subkey will not
  * be searched at all.
+ *
+ * Note: getNames() : uses ConfSimple method, this does *not* inherit 
+ *     names from englobing submaps.
  */
 class ConfTree : public ConfSimple {
 
@@ -280,7 +285,7 @@ public:
      * expansion */
     ConfTree(const char *fname, int readonly = 0) 
 	: ConfSimple(fname, readonly, true) {}
-    ConfTree(string *data, int readonly = 0)
+    ConfTree(const string &data, int readonly = 0)
 	: ConfSimple(data, readonly, true) {}
     ConfTree(int readonly = 0)
 	: ConfSimple(readonly, true) {}
@@ -296,7 +301,7 @@ public:
      * parents.
      * @return 0 if name not found, 1 else
      */
-    virtual int get(const string &name, string &value, const string &sk);
+    virtual int get(const string &name, string &value, const string &sk) const;
 };
 
 /** 
@@ -353,9 +358,9 @@ public:
 	return *this;
     }
 
-    virtual int get(const string &name, string &value, const string &sk) 
+    virtual int get(const string &name, string &value, const string &sk) const
     {
-	typename list<T*>::iterator it;
+	typename list<T*>::const_iterator it;
 	for (it = m_confs.begin();it != m_confs.end();it++) {
 	    if ((*it)->get(name, value, sk))
 		return true;
@@ -363,7 +368,18 @@ public:
 	return false;
     }
 
-    virtual int set(const string &nm, const string &val, const string &sk = string()) 
+    virtual bool hasNameAnywhere(const string& nm)
+    {
+	typename list<T*>::iterator it;
+	for (it = m_confs.begin();it != m_confs.end();it++) {
+	    if ((*it)->hasNameAnywhere(nm))
+		return true;
+	}
+	return false;
+    }
+
+    virtual int set(const string &nm, const string &val, 
+                    const string &sk = string()) 
     {
 	if (!m_ok)
 	    return 0;
@@ -405,13 +421,13 @@ public:
 	return m_confs.front()->holdWrites(on);
     }
 
-    virtual list<string> getNames(const string &sk) 
+    virtual list<string> getNames(const string &sk, const char *pattern = 0)
     {
 	list<string> nms;
 	typename list<T*>::iterator it;
 	for (it = m_confs.begin();it != m_confs.end(); it++) {
 	    list<string> lst;
-	    lst = (*it)->getNames(sk);
+	    lst = (*it)->getNames(sk, pattern);
 	    nms.insert(nms.end(), lst.begin(), lst.end());
 	}
 	nms.sort();
@@ -433,7 +449,7 @@ public:
 	return sks;
     }
 
-    virtual bool ok() {return m_ok;}
+    virtual bool ok() const {return m_ok;}
 
 private:
     bool     m_ok;
