@@ -98,9 +98,9 @@ bool ExecCmd::which(const string& cmd, string& exepath, const char* path)
     if (pp == 0)
 	return false;
 
-    list<string> pels;
+    vector<string> pels;
     stringToTokens(pp, pels, ":");
-    for (list<string>::iterator it = pels.begin(); it != pels.end(); it++) {
+    for (vector<string>::iterator it = pels.begin(); it != pels.end(); it++) {
 	if (it->empty())
 	    *it = ".";
 	string candidate = (it->empty() ? string(".") : *it) + "/" + cmd;
@@ -462,9 +462,9 @@ int ExecCmd::wait(bool haderror)
     if (!m_killRequest && m_pid > 0) {
 	if (waitpid(m_pid, &status, 0) < 0) 
 	    status = -1;
+        LOGDEB(("ExecCmd::wait: got status 0x%x\n", status));
 	m_pid = -1;
     }
-    LOGDEB(("ExecCmd::wait: got status 0x%x\n", status));
     return haderror ? -1 : status;
 }
 
