@@ -19,27 +19,13 @@ static char rcsid[] = "@(#$Id: pathut.cpp,v 1.23 2008-11-24 15:47:40 dockes Exp
  */
 
 #ifndef TEST_PATHUT
+#include "autoconfig.h"
+
 #include <unistd.h>
 #include <fcntl.h>
 #include <sys/param.h>
 #include <pwd.h>
 #include <math.h>
-#include <cstdlib>
-#include <cstring>
-
-#include <iostream>
-#include <list>
-#include <stack>
-#ifndef NO_NAMESPACES
-using std::string;
-using std::list;
-using std::stack;
-#endif /* NO_NAMESPACES */
-
-#include "autoconfig.h"
-#include "pathut.h"
-#include "transcode.h"
-
 #include <sys/types.h>
 // Let's include all files where statfs can be defined and hope for no
 // conflict...
@@ -56,6 +42,21 @@ using std::stack;
 #include <sys/vfs.h>
 #endif
 
+#include <cstdlib>
+#include <cstring>
+#include <iostream>
+#include <list>
+#include <stack>
+#ifndef NO_NAMESPACES
+using std::string;
+using std::list;
+using std::stack;
+#endif /* NO_NAMESPACES */
+
+#include "pathut.h"
+#include "transcode.h"
+#include "wipedir.h"
+
 bool fsocc(const string &path, int *pc, long *blocks)
 {
 #ifdef sun
@@ -153,7 +154,7 @@ TempFileInternal::TempFileInternal(const string& suffix)
     int fd;
     if ((fd = mkstemp(cp)) < 0) {
 	free(cp);
-	m_reason = "maketmpdir: mkstemp failed\n";
+	m_reason = "TempFileInternal: mkstemp failed\n";
 	return;
     }
     close(fd);
@@ -175,6 +176,34 @@ TempFileInternal::~TempFileInternal()
 	unlink(m_filename.c_str());
 }
 
+TempDir::TempDir()
+{
+    if (!maketmpdir(m_dirname, m_reason)) {
+	m_dirname.erase();
+	return;
+    }
+}
+
+TempDir::~TempDir()
+{
+    if (!m_dirname.empty()) {
+	(void)wipedir(m_dirname, true, true);
+	m_dirname.erase();
+    }
+}
+
+bool TempDir::wipe()
+{
+    if (m_dirname.empty()) {
+	m_reason = "TempDir::wipe: no directory !\n";
+	return false;
+    }
+    if (wipedir(m_dirname, false, true)) {
+	m_reason = "TempDir::wipe: wipedir failed\n";
+	return false;
+    }
+    return true;
+}
 
 void path_catslash(string &s) {
     if (s.empty() || s[s.length() - 1] != '/')
@@ -302,10 +331,10 @@ extern string path_canon(const string &is)
 	}
 	s = path_cat(string(buf), s); 
     }
-    list<string>elems;
+    vector<string> elems;
     stringToTokens(s, elems, "/");
-    list<string> cleaned;
-    for (list<string>::const_iterator it = elems.begin(); 
+    vector<string> cleaned;
+    for (vector<string>::const_iterator it = elems.begin(); 
 	 it != elems.end(); it++){
 	if (*it == "..") {
 	    if (!cleaned.empty())
@@ -317,7 +346,7 @@ extern string path_canon(const string &is)
     }
     string ret;
     if (!cleaned.empty()) {
-	for (list<string>::const_iterator it = cleaned.begin(); 
+	for (vector<string>::const_iterator it = cleaned.begin(); 
 	     it != cleaned.end(); it++) {
 	    ret += "/";
 	    ret += *it;
